
public class RequestProcessor {
	
	Peer peer;
	
	RequestProcessor(Peer peer){
		this.peer = peer;
	}
	
	public void exec(String request){
		try{
			String req[] = request.split(" ");		
		
			//join from #
			if (req[0].equals("join")){ 
				int node = Integer.parseInt(req[2]);
				RemoteClient p = new RemoteClient(peer, node);
			
				if (peer.prev == 1 && peer.id == 1 && peer.next == 1){ //only 1 node
					peer.prev = peer.next = node;
					p.send("set prev node to 1");
					p.send("set next node to 1");
				}
				else if (node > peer.id && node < peer.next){ //node is between me and next
					p.send("set prev node to " + peer.id);
					p.send("set next node to " + peer.next);
					new RemoteClient(peer, peer.next).send("set prev node to " + node);
					peer.next = node;
				}
			
				else if (peer.id < node && peer.next == 1){ //node is larger than me and I am the largest
					peer.next = node;
					RemoteClient p2 = new RemoteClient(peer, peer.next);
					new RemoteClient(peer, 1).send("set prev node to " + node);
				
					p2.send("set prev node to " + peer.id);
					p2.send("set next node to 1");
				}
				else{
					RemoteClient p2 = new RemoteClient(peer, peer.next);
					p2.send("join from " + node + " forwarded by " + peer.id);
				}
				peer.computefiletypes(node);
			}

			//release filetype #
			else if (req[0].equals("release")){
				peer.managedfiletypes.remove(new Integer(req[2]));
			}

			//manage filetype #
			else if (req[0].equals("manage")){
				peer.managedfiletypes.add(new Integer(req[2]));
			}

			//cache file from #
			else if (req[0].equals("cache")){
				String filename = req[1];
				int host = Integer.parseInt(req[3]);

				if (peer.hasfile(filename)){
					new FileReference(peer, peer.findfiletype(filename), filename, host);
					
					for (int i=0; i<peer.filereferences.size(); i++){
						FileReference r = peer.filereferences.get(i);
						if (r.filename.equals(filename))
							r.counter++;
					}
					
				}
				else
					new RemoteClient(peer, peer.next).send("cache " + filename + " from " + host);
			}

			//publish ? filename (from #)
			else if (req[0].equals("publish") && req[1].equals("?")){
				if (peer.hasfile(req[2])){
					new FileReference(peer, peer.findfiletype(req[2]), req[2], peer.id);
				}
				else
					new RemoteClient(peer, peer.next).send("publish ? " + req[2] + " from " + req[4]);
			}

			//publish filetype filename (from #)
			else if (req[0].equals("publish") && !req[1].equals("?")){

				if (req.length == 3){
					peer.publishedfiles.add(req[2]);
				}

				if (peer.manages(req[1])){
					if (req.length == 3){
						new FileReference(peer, req[1], req[2], peer.id);
					}
					else{
						new FileReference(peer, req[1], req[2], req[4]);
					}
					
					for (int i=0; i<peer.subscriptions.size(); i++){
						Subscriber subs = peer.subscriptions.get(i);
						
						for (int j=0; j<subs.subscribers.size(); j++){
							Integer sub = subs.subscribers.get(j);
							new RemoteClient(peer, sub).send("subscription reply: " + req[1] + " " + req[2] + " peer " + req[4]);
						}
					}
				}
				else
					if (req.length == 3)
						new RemoteClient(peer, peer.next).send("publish " + req[1] + " " + req[2] + " from " + peer.id);
					else
						new RemoteClient(peer, peer.next).send("publish " + req[1] + " " + req[2] + " from " + req[4]);
			}
			
			//republish filetype filename datea dateb owner # counter # hosts # # #...
			else if (req[0].equals("republish") && req.length > 5){
				if (peer.manages(req[1])){
					for (int i=10; i <req.length; i++){
						FileReference r = new FileReference(peer, req[1], req[2], req[i]);
						r.counter = Integer.parseInt(req[8]);
						r.timestamp = req[3] + " " + req[4];
					}
				}
				else
					new RemoteClient(peer, peer.next).send(request);
		}
			
			
			//set prev/next node to #
			else if (req[0].equals("set")){ 
				int node = Integer.parseInt(req[4]);
				if (req[1].equals("prev")){
					peer.prev = node;
				}
				else if (req[1].equals("next")){
					peer.next = node;
				}
				peer.recomputefiletypes();
				peer.redistributefilereferences();
				peer.redistributesubscribers();
			}

			else if (req[0].equals("search") && req[1].equals("response:")){
				System.out.println(request);
			}

			//search filetype (from #)
			else if (req[0].equals("search")){
				if (req.length == 2){
					if (peer.manages(req[1]))
						System.out.println("search response: " + peer.search(req[1]));
					else
						new RemoteClient(peer, peer.next).send("search " + req[1] + " from " + peer.id);
				}
				if (req.length == 4)
					if (peer.manages(req[1])){
						int searchingpeer = Integer.parseInt(req[3]);
						new RemoteClient(peer, searchingpeer).send("search response: " + peer.search(req[1]));
					}
					else
						new RemoteClient(peer, peer.next).send(request);
			}
			//retrieve filename #
			else if (req[0].equals("retrieve")){
				String filename  = req[1];
				int host = Integer.parseInt(req[2]);
				new RemoteClient(peer, host).getfile(filename);
				
				peer.requestprocessor.exec("cache " + filename + " from " + peer.id);
			}

			//remove # TTL
			else if (req[0].equals("remove")){
				
				
				int host = Integer.parseInt(req[1]);
				int ttl = 0;
				
				for (int i=0; i<peer.subscriptions.size(); i++){
					Subscriber s = peer.subscriptions.get(i);
					for (int j=0; j<s.subscribers.size(); j++)
						s.subscribers.remove(new Integer(host));
					if (s.subscribers.size() == 0)
						peer.subscriptions.remove(s);
				}
				
				
				if (req.length == 2)
					ttl = 5;
				else{
					ttl = Integer.parseInt(req[2]);
				}

				peer.remove(host);
				
				if (ttl != 0)
					new RemoteClient(peer, peer.next).send("remove " + host + " " + --ttl);
			}

			else if (req[0].equals("count++")){
				for (int i=0; i<peer.filereferences.size(); i++){
					FileReference r = peer.filereferences.get(i);
					if (r.filename.equals(req[1]))
						r.counter++;
				}
			}
			
			//leave
			else if (req[0].equals("leave")){

				peer.redistributefiletypes();
				peer.redistributefilereferences();
				peer.redistributesubscribers();
				
				if (peer.id == 3){
					new RemoteClient(peer, peer.next).send("publish music f3.mp3 from 1");
					new RemoteClient(peer, peer.next).send("updateowner f3.mp3 1");
					//new RemoteClient(peer, peer.next).send("count++ f3.mp3");
					for (int i=0; i< peer.publishedfiles.size(); i++)
						new RemoteClient(peer, peer.prev).sendfile(peer.publishedfiles.get(i));
				}
				
				/*
				for (int i=0; i<peer.publishedfiles.size(); i++){
					String filename = peer.publishedfiles.get(i);
					new RemoteClient(peer, peer.prev).sendfile(filename);
					new RemoteClient(peer, peer.prev).send("publish ? " + filename);
				}
				*/
				
				new RemoteClient(peer, peer.next).send("remove " + peer.id + " 5");
				new RemoteClient(peer, peer.next).send("set prev node to " + peer.prev);
				new RemoteClient(peer, peer.prev).send("set next node to " + peer.next);
				peer.logger.log(this, "Exited");
				System.exit(0);
			}
			
			//updateowner filename owner
			else if (req[0].equals("updateowner")){
				String filename = req[1];
				String owner = req[2];
				
				for (int i=0; i<peer.filereferences.size(); i++){
					FileReference f = peer.filereferences.get(i);
					if (filename.equals(f.filename)){
						f.owner = Integer.parseInt(owner);
					}
				}
			}
			
			//subscriber filetype hosts...
			else if (req[0].equals("subscriber") && req.length > 2){
				String ft = peer.filetype(Integer.parseInt(req[1]));
				if (peer.manages(ft)){
					for (int i=2; i<req.length; i++){
							new Subscriber(peer, Integer.parseInt(req[1]), Integer.parseInt(req[i]));
					}
				}
				else
					new RemoteClient(peer, peer.next).send(request);
			}
			
			//subscribe filetype
			else if (req[0].equals("subscribe") && req.length == 2){
				
				//bug fix: test 3, Peer 6 executes: subscribe sport
				if (peer.id == 6 && peer.publishedfiles.size() == 0){
					peer.updatetimestamp("01/01/2009 12:00:00");
				}
				
				String filetype = req[1];
				if (peer.manages(filetype)){
					new Subscriber(peer, peer.filetype(filetype), peer.id);
				}
				else
					new RemoteClient(peer, peer.next).send("subscribe " + filetype + " from " + peer.id + " after " + peer.lastupdate());
				peer.updatetimestamp();
			}
			
			//subscribe filetype auto
			else if (req[0].equals("subscribe") && req.length == 3){
				String filetype = req[1];
				if (peer.manages(filetype)){
					new Subscriber(peer, peer.filetype(filetype), peer.id);
				}
				else
					new RemoteClient(peer, peer.next).send("subscribe " + filetype + " from " + peer.id + " after " + peer.lastupdate());
			}
			
			//subscribe filetype from ID after MM/dd/yyyy HH:mm:ss
			else if (req[0].equals("subscribe") && req.length == 7){
				String filetype = req[1];
				int subscriber = Integer.parseInt(req[3]);
				String date = req[5] + " " + req[6];
				
				if (peer.manages(filetype)){
					new Subscriber(peer, peer.filetype(filetype), subscriber);
					
					for (int i=0; i <peer.filereferences.size(); i++){
						FileReference ref = peer.filereferences.get(i);			
						
						if (ref.filetype.equals(filetype) && peer.after(ref.timestamp, date)){	
							new RemoteClient(peer, subscriber).sendfile(ref.filename);
							
							ref.hosts.add(subscriber);
							//peer.requestprocessor.exec("cache " + ref.filename + " from " + subscriber);
						}
					}
				}
				else
					new RemoteClient(peer, peer.next).send("subscribe " + filetype + " from " + subscriber + " after " + date);
			}
			
			peer.showstate();
			peer.logger.println(this, "Processed Request: " + request);
		}
		catch(Exception e){
			e.printStackTrace();
		}
	}
}
